@@ -113,6 +113,7 @@ itAlloc(void)
 	FALSE,			/* boolean_t itString */
 	FALSE,			/* boolean_t itVarArray */
 	FALSE,			/* boolean_t itIndefinite */
+	FALSE,			/* boolean_t itKernelPort */
 	itNULL,			/* ipc_type_t *itElement */
 	strNULL,		/* identifier_t itUserType */
 	strNULL,		/* identifier_t itServerType */
@@ -154,7 +155,7 @@ itCalculateSizeInfo(ipc_type_t *it)
     if (it->itInLine)
     {
 	u_int bytes = (it->itNumber * it->itSize + 7) / 8;
-	u_int padding = (4 - bytes%4)%4;
+	u_int padding = (word_size - bytes % word_size) % word_size;
 
 	it->itTypeSize = bytes;
 	it->itPadSize = padding;
@@ -219,16 +220,19 @@ itCalculateNameInfo(ipc_type_t *it)
 	(((it->itInName == MACH_MSG_TYPE_POLYMORPHIC) &&
 	  (it->itOutName == MACH_MSG_TYPE_POLYMORPHIC)) ||
 	 MACH_MSG_TYPE_PORT_ANY(it->itInName) ||
-	 MACH_MSG_TYPE_PORT_ANY(it->itOutName)))
+	 MACH_MSG_TYPE_PORT_ANY(it->itOutName))) {
 	it->itServerType = "ipc_port_t";
-
-    if (IsKernelUser &&
+        it->itKernelPort = TRUE;
+    } else if (IsKernelUser &&
 	streql(it->itUserType, "mach_port_t") &&
 	(((it->itInName == MACH_MSG_TYPE_POLYMORPHIC) &&
 	  (it->itOutName == MACH_MSG_TYPE_POLYMORPHIC)) ||
 	 MACH_MSG_TYPE_PORT_ANY(it->itInName) ||
-	 MACH_MSG_TYPE_PORT_ANY(it->itOutName)))
+	 MACH_MSG_TYPE_PORT_ANY(it->itOutName))) {
 	it->itUserType = "ipc_port_t";
+        it->itKernelPort = TRUE;
+    } else
+        it->itKernelPort = FALSE;
 
     if (it->itTransType == strNULL)
 	it->itTransType = it->itServerType;
